let rootUrl = 'http://' + window.location.hostname + ':8080/';
let dmnApi = rootUrl + 'api/dmn/';
let inactiveBackend = true;

$(document).ready(function () {
  // load types
  loadAvailableTypes();
});
/**
 * Global definition for verification results
 * @type {VerifierResultSet}
 */
let verifierResults = {};

/**
 *
 * @type {Array.<VerificationType>}
 */
let types = [];

let $verifierTypes;
let lastTypeSelect = undefined;
let $select = $(`
    <select name="verifier" id="verifier"
      class="dmn-verifier-header-item dmn-verifier-select clickable"
      title="Select calculated verification"
      onchange="renderVerifierResult(this)">
  `);

function handleStatus() {
  let $dot = $('#backend-status');
  if (inactiveBackend) {
    verifierResults = {};
    $dot.removeClass('active');
    $dot.addClass('inactive');
    alert(
        "Error: The backend is currently not available. Please try again later.")
  } else {
    $dot.removeClass('inactive');
    $dot.addClass('active');
  }
  cleanDmnVerifierRoot();
}

function loadAvailableTypes() {
  $.ajax({
    timeout: 1000,
    url: dmnApi + 'verification/types',
    type: 'GET',
    contentType: 'text/xml',
    error: function (err) {
      inactiveBackend = true;
      handleStatus();
    },
    success: function (data) {
      inactiveBackend = false;
      types = data;
      // sort types by classification and name
      types.sort(function (
          /** VerificationType */ firstEl, /** VerificationType */ secondEl) {
        if (firstEl.classification.name
            === secondEl.classification.name) {
          if (firstEl.niceName === secondEl.niceName) {
            return 0;
          }
          return firstEl.niceName < secondEl.niceName ? -1 : 1;
        } else if (firstEl.classification.name
            < secondEl.classification.name) {
          return -1;
        } else {
          return 1;
        }
      });
      $verifierTypes = renderTypeOptions();
      handleStatus();
    }
  });
}

function cleanDmnVerifierRoot() {
  let $root = $('#root-dmn-verifier').empty();
  let $header = $(`      
      <div id="dmn-verifier-header">
  `);
  // check, if backend is available
  if ($verifierTypes === undefined) {
    $header.append($(`
      <button class="clickable dmn-verifier-header-item" id="dmn-button-reload-verifier" 
      title="Reconnect backend" onClick="loadAvailableTypes()">
      Reconnect verification backend</button>
    `));
  } else {
    $header.append($verifierTypes);
    $verifierTypes.select2({
      containerCssClass: "dmn-verifier-header-item dmn-verifier-select clickable",
      placeholder: {
        id: -1,
        text: "All verifier"
      },
      allowClear: true
    });
    // Add Button
    $header.append($(`
      <button class="clickable dmn-verifier-header-item" id="dmn-button-verify" 
      title="Request verifications" onClick="checkVerifications()">
      Verify</button>
    `));
    $select.empty();
    $header.append($select);
  }

// add header to root
  $root.append($header);
  $root.append($(`<div id="dmn-verifier-content">`));
  return $root;
}

function checkVerifications() {
  cleanDmnVerifierRoot();
  getVerifications();
}

function getVerifications() {
  dmnModeler.saveXML({format: true}, function (err, xml) {
    if (err) {
      return alert(err);
    } else {
      console.log('Requesting dmn verifications..');
    }
    let apiPath = dmnApi + 'verification';
    // check, if a verifier is preselected
    if ($verifierTypes[0].selectedOptions[0] !== undefined) {
      apiPath += "/types";
      for (let i = 0; i < $verifierTypes[0].selectedOptions.length; i++) {
        if (i === 0) {
          apiPath += '?';
        } else {
          apiPath += '&';
        }
        apiPath += 'typeName=' + $verifierTypes[0].selectedOptions[i].value;
      }
    }
    $.ajax({
      timeout: 0, // ??
      url: apiPath,
      type: 'POST',
      contentType: 'text/xml',
      data: xml,

      error: function (err) {
        alert(
            'Error: There was a fatal error while parsing the xml of the dmn.');
      },
      success: function (data) {
        console.log('successful dmn verification request:');
        console.log(data);
        verifierResults = data;
        // sort the list 1. classification, 2. size of entries
        verifierResults.verifier.sort(function (
            /** VerifierResult */ firstEl, /** VerifierResult */ secondEl) {
          if (firstEl.type.classification.name
              === secondEl.type.classification.name) {
            if (firstEl.size === secondEl.size) {
              return firstEl.type.name < secondEl.type.name ? -1 : 1;
            } else if (firstEl.size < secondEl.size) {
              return 1;
            } else {
              return -1;
            }
          } else if (firstEl.type.classification.name
              < secondEl.type.classification.name) {
            return -1;
          } else {
            return 1;
          }
        });
        renderDmnVerifierOptions();
      }
    });
  });
}

function renderTypeOptions() {
  let $select = $(`
    <select name="verifier[]" id="dmn-verifier-types" multiple="multiple"
    title="Preselect verifications"
      class="dmn-verifier-header-item dmn-verifier-select clickable">
  `);
  let currentOpt = '';
  let $curGroup;
  for (let i = 0; i < types.length; i++) {
    let bolNewGroup = currentOpt
        !== types[i].classification.niceName;
    if (bolNewGroup) {
      if (currentOpt === currentOpt) {
        $select.append($curGroup);
      }
      $curGroup = ($(
          `<optgroup label="${types[i].classification.niceName}">
      `));
      currentOpt = types[i].classification.niceName;
    }
    $curGroup.append($(`
      <option value="${types[i].name}">${types[i].niceName}</option>`
    ));

  }
  $select.append($curGroup);
  return $select;
}

function renderDmnVerifierOptions() {
  $select.append($(`<option>Select a verifier</option>`));
  let currentOpt = '';
  let $curGroup;
  let selOpt = false;
  for (let i = 0; i < verifierResults.verifier.length; i++) {
    let verifier = verifierResults.verifier[i];
    let bolNewGroup = currentOpt
        !== verifier.type.classification.niceName;
    if (bolNewGroup) {
      if (currentOpt === currentOpt) {
        $select.append($curGroup);
      }
      $curGroup = ($(`
          <optgroup label="${verifier.type.classification.niceName}">
      `));
      currentOpt = verifier.type.classification.niceName;
    }
    let $option;
    if (verifierResults.verifier.length === 1 ||
        lastTypeSelect !== undefined && lastTypeSelect === verifier.type.name) {
      selOpt = true;
      $option = $(`
        <option value="${verifier.type.name}" selected>${verifier.type.niceName} (${verifier.size})</option>
      `);
    } else {
      $option = $(`
        <option value="${verifier.type.name}">${verifier.type.niceName} (${verifier.size})</option>
      `);
    }
    if (verifier.size === 0) {
      $option.css('color', '#999')
    }
    $curGroup.append($option);
  }
  $select.append($curGroup);

  if (selOpt) {
    renderVerifierResult($select[0].selectedOptions[0]);
  }
}

/**
 *
 * @param verifierSelect
 * @Type option
 */
function renderVerifierResult(verifierSelect) {
  let verifier = findVerifierByName(verifierSelect.value);
  if (verifier === undefined) {
    lastTypeSelect = undefined;
  } else {
    lastTypeSelect = verifier.type.name;
    renderVerifier(verifier);
  }
}

/**
 *
 * @param verifier
 * @Type VerifierResult
 */
function renderVerifier(verifier) {
  let $verifierContent = $('#dmn-verifier-content');
  $verifierContent.empty();
  $verifierContent.css('display', 'none');
  let $verifierType = $(`<div class="verifier-type"></div>`);
  let $verifierEntries = $(`<div class="verifier-entries"></div>`);

  // Add name and description of verifier
  $verifierType.append($(`
    <div class="verifier-type-name"><h2>${verifier.type.niceName}</h2></div>
  `));
  $verifierType.append($(`
    <div class="verifier-type-description"><p>${verifier.type.description}</p></div>
  `));

  // Add entries if min 1 result
  if (verifier.entries.length > 0) {
    for (let i = 0; i < verifier.entries.length; i++) {
      $verifierEntries.append(renderVerificationEntry(verifier.entries[i]));
    }
  }
  $verifierContent.append($verifierType);
  $verifierContent.append($verifierEntries);
  $verifierContent.fadeIn(250);
}

/**
 *
 * @param verificationEntry
 * @Type VerificationEntry
 */
function renderVerificationEntry(verificationEntry) {
  let $entryContainer = $(`<div class="verification-container"></div>`);
  $entryContainer.append($(`
      <span class="dmn-verification-icon dmn-${verificationEntry.verificationClassification}"/>
  `));
  $entryContainer.append($(`
      <div class="verification-message">${verificationEntry.message}</div>
  `));
  // "dmn-${verificationEntry.verificationClassification}"
  let $fixButtons = $(`<div class="verification-container-fix-buttons"></div>`);

  /*
  $fixButtons.append(
      $(`<button class="verification-fix-button clickable">Show</button>`));
  $fixButtons.append(
      $(`<button class="verification-fix-button clickable">Fix</button>`));
  $fixButtons.append(
      $(`<button class="verification-fix-button clickable">Fix</button>`)); */
  $entryContainer.append($fixButtons); // TODO: render buttons
  return $entryContainer;
}

/**
 *
 * @param verifierName
 * @type string
 * @return VerifierResult
 */
function findVerifierByName(verifierName) {
  for (let i = 0; i < verifierResults.verifier.length; i++) {
    if (verifierResults.verifier[i].type.name === verifierName) {
      return verifierResults.verifier[i];
    }
  }
  return undefined;
}
